import unittest
import pygdrive

import unittest.mock as mock

CREDENTIALS_FILEPATH = {}
SCOPES = []
DOWNLOAD_DIR = ''
UPLOAD_DIR = ''


class TestBasic(unittest.TestCase):
    """
    Base class
    """
    @mock.patch.object(pygdrive.gdrive, 'build', autospec=True)
    @mock.patch.object(pygdrive.gdrive.service_account.Credentials, 'from_service_account_file', autospec=True)
    def setUp(self, mock_credential_object, mock_build_object):
        self.test_object = pygdrive.gdrive.Gdrive(CREDENTIALS_FILEPATH, SCOPES, DOWNLOAD_DIR, UPLOAD_DIR)

    def tearDown(self):
        self.test_object = None


class TestDrive(TestBasic):
    def test_object_create(self):
        self.assertTrue(self.test_object is not None)


class TestFolder(TestBasic):
    def test_find_folder(self):
        mock_id, mock_name = 'test_id123', 'test_name'
        self.test_object.service.files.return_value.list.return_value.execute.return_value = {
            'files': [{'id': mock_id, 'name': mock_name}]}
        id, name = self.test_object.find_folder('test_file')
        self.assertEqual(mock_id, id)

    def test_folder_create(self):
        mock_parent_id = 'parent_id123'
        mock_folder_id, mock_folder_name = 'test_id123', 'folder_test'
        self.test_object.service.files.return_value.create.return_value.execute.return_value = {'id': mock_folder_id}
        folder_id = self.test_object.create_folder(mock_parent_id, mock_folder_name)
        self.assertEqual(folder_id, mock_folder_id)


class TestUploading(TestBasic):
    @mock.patch.object(pygdrive.gdrive, 'MediaFileUpload', autospec=True)
    def test_object_upload(self, mock_media_upload):
        mock_parent_id = 'parent_id123'
        mock_file_id, mock_file_name = 'test_id123', 'file_test2.csv'
        self.test_object.service.files.return_value.create.return_value.execute.return_value = {'id': mock_file_id}
        file_id = self.test_object.upload_file(mock_parent_id, mock_file_name)
        self.assertEqual(file_id, mock_file_id)


# class TestDownloading(TestBasic):
#     def test_object_download(self):
#         mock_file_id, mock_file_name = 'test_id123', 'file_test2.csv'
#         self.test_object.service.files.return_value.get_media.return_value.execute.return_value = None
#         return_value = self.test_object.download_file(mock_file_id, mock_file_name)
#         self.assertEqual(None, return_value)


class TestFileSearching(TestBasic):
    def test_search_existing_file(self):
        mock_file_id, mock_file_name = 'test_id123', 'file_test2.csv'
        self.test_object.service.files.return_value.list.return_value.execute.return_value = {
            'files': [{'id': mock_file_id, 'name': mock_file_name}]}
        id, name = self.test_object.find_file(mock_file_name)
        self.assertEqual(id, mock_file_id)

    def test_search_existing_folder(self):
        mock_folder_id, mock_folder_name = 'test_id123', 'file_test2.csv'
        self.test_object.service.files.return_value.list.return_value.execute.return_value = {
            'files': [{'id': mock_folder_id, 'name': mock_folder_name}]}
        id, name = self.test_object.find_folder(mock_folder_name)
        self.assertEqual(id, mock_folder_id)



from setuptools import setup

setup(
    name='pygdrive',
    version='1.0.1',
    packages=['pygdrive', 'pygdrive.tests'],
    url='https://github.com/vectorspace-ai/pygdrive',
    license='LICENSE',
    author='Vectorspace AI',
    author_email='arina@vectorspace.ai',
    description='Google Drive API for service accounts',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        "google-api-python-client>=2.1.0",
        "google-auth-httplib2>=0.1.0",
        "google-auth-oauthlib>=0.4.4",
    ],
)



from google.oauth2 import service_account
from googleapiclient.discovery import build
from os import path
from googleapiclient.http import MediaFileUpload


class Gdrive:
    """ A class represent google drive. """

    def __init__(self, credentials_filepath, scopes, download_dir, upload_dir):
        """
        :str credentials_filepath: The path to the service account json file.
        :str scopes:  User-defined scopes to request during the
                authorization grant.
        :str download_path: The path to the download folder.
        :str upload_path: The path to the upload folder.
        """
        self.service = None
        self.credentials_filepath = credentials_filepath
        self.scopes = scopes
        self.download_dir = download_dir
        self.upload_dir = upload_dir
        self.connect()

    def connect(self):
        """
        Creates a Credentials instance from a service account json file.
        :return: None
        """
        credentials = service_account.Credentials.from_service_account_file(
            self.credentials_filepath, scopes=self.scopes)
        self.service = build('drive', 'v3', credentials=credentials)

    def check_does_folder_exist(self, folder_name):
        """
        Checking if folder with specified name exist.
        :str folder_name: Name of the folder which should be search.
        :return: True if folder found. Otherwise False.
        """
        folder_search = "mimeType = 'application/vnd.google-apps.folder'"
        results = self.service.files().list(fields="nextPageToken, files(id, name)", orderBy='createdTime',
                                            q=f"name contains '{folder_name}' and {folder_search}").execute()
        item = results.get('files', [])
        if item:
            return True
        return False

    def check_does_file_exist(self, file_name):
        """
        Checking if file with specified name exist.
        :str file_name: Name of the file which should be search.
        :return: True if file found. Otherwise False.
        """
        file_search = "mimeType != 'application/vnd.google-apps.folder'"
        results = self.service.files().list(fields="nextPageToken, files(id, name)", orderBy='createdTime',
                                            q=f"name = '{file_name}' and {file_search}").execute()
        item = results.get('files', [])
        if item:
            return True
        return False

    def find_folder(self, folder_name):
        """
        Searching for folder name.
        :str folder_name: Name of the folder which should be search.
        :return: id, name of the founded folder.
        """
        folder_search = "mimeType = 'application/vnd.google-apps.folder'"
        results = self.service.files().list(fields="nextPageToken, files(id, name)", orderBy='createdTime',
                                            q=f"name = '{folder_name}' and {folder_search}").execute()
        item = results.get('files', [])
        return item[-1]['id'], item[-1]['name']

    def find_file(self, file_name):
        """
        Searching for file name.
        :str file_name: Name of the file which should be search.
        :return: id, name of the founded folder.
        """
        file_search = "mimeType != 'application/vnd.google-apps.folder'"
        results = self.service.files().list(fields="nextPageToken, files(id, name)", orderBy='createdTime',
                                            q=f"name = '{file_name}' and {file_search}").execute()
        item = results.get('files', [])
        return item[-1]['id'], item[-1]['name']

    def find_file_in_folder(self, folder_id):
        """
        Searching for last files sorted by createdTime inside folder.
        :str folder_id: : Id of the folder.
        :return: id, name of the founded file.
        """
        file_search = "mimeType != 'application/vnd.google-apps.folder'"
        results = self.service.files().list(fields="nextPageToken, files(id, name)", orderBy='createdTime',
                                            q=f"{folder_id} in parents and {file_search}").execute()
        item = results.get('files', [])
        return item[-1]['id'], item[-1]['name']

    def download_file(self, file_id, file_name):
        """
        Download file by file_id.
        :str file_id: Id of the file.
        :str file_name: Name of the file.
        :return: None
        """
        file_body = self.service.files().get_media(fileId=file_id).execute()
        with open(path.join(self.download_dir, file_name), "wb") as wer:
            wer.write(file_body)

    def create_folder(self, parent_id, folder_name):
        """
         Create folder inside parent folder.
        :str parent_id: Parent folder id.
        :str folder_name: Folder name.
        :return: id of the folder
        """

        file_metadata = {
            'name': folder_name,
            'mimeType': 'application/vnd.google-apps.folder',
            'parents': [parent_id]
        }
        file = self.service.files().create(body=file_metadata,
                                           fields='id').execute()
        return file['id']


    def upload_file(self, folder_id, file_name):
        """
        Upload file to folder with folder_id.
        :str folder_id: Parent id.
        :str file_name: File name.
        :return: id of the file
        """

        file_metadata = {
            'name': file_name,
            'parents': [folder_id]
        }
        media = MediaFileUpload(path.join(self.upload_dir, file_name),
                                resumable=True)
        file = self.service.files().create(body=file_metadata,
                                           media_body=media,
                                           fields='id').execute()
        return file['id']

    # def delete_file(self, file_id):
    #     """
    #     delete file or folder by id
    #     :int file_id: Id of the file to be deleted
    #     :return: None
    #     """
    #     self.service.files().delete(fileId=file_id).execute()
